/************************************************************************************
TerraLib - a library for developing GIS applications.
Copyright  2001-2004 INPE and Tecgraf/PUC-Rio.

This code is part of the TerraLib library.
This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

You should have received a copy of the GNU Lesser General Public
License along with this library.

The authors reassure the license terms regarding the warranties.
They specifically disclaim any warranties, including, but not limited to,
the implied warranties of merchantability and fitness for a particular purpose.
The library provided hereunder is on an "as is" basis, and the authors have no
obligation to provide maintenance, support, updates, enhancements, or modifications.
In no event shall INPE and Tecgraf / PUC-Rio be held liable to any party for direct,
indirect, special, incidental, or consequential damages arising out of the use
of this library and its documentation.
*************************************************************************************/

/**
 * @file TePostgreSQL.cpp
 * @brief This file contains the especifics routines of SQLite.
 * @author Joo Borges <joaoborges@mossol.com>    
 */

/**
 * PS.: For this code works properly, you must to compile SQLite with 
 * SQLITE_ENABLE_COLUMN_METADATA preprocessor symbol defined, one way to do this is add 
 * the following line in the file sqliteInt.h, in the src directory, before compiling.
 * 
 * #define SQLITE_ENABLE_COLUMN_METADATA
 * 
 */

#include "TeSQLite.h"
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>

extern "C" {
extern char *strxmov(char* dest, const char* src, ...);
}

#ifndef MAX
  #define MAX(a,b) ((a>b)?a:b)
#endif
#ifndef MIN
  #define MIN(a,b) ((a<b)?a:b)
#endif

double swappd2(double din)
{
	char  stri[8],stro[8];
	double dout;

	memcpy (stri,&din,8);
	stro[0] = stri[7];
	stro[1] = stri[6];
	stro[2] = stri[5];
	stro[3] = stri[4];
	stro[4] = stri[3];
	stro[5] = stri[2];
	stro[6] = stri[1];
	stro[7] = stri[0];
	memcpy (&dout,stro,8);
	return (dout);
}

/**
 * Constructor
 */
TeSQLite::TeSQLite()
{
	isConnected_ = false;
	dbmsName_ = "SQLite";
	bufferBlob_ = 0;
	bufferBlobSize_ = 0;
    sqliteDir = "";
    dbHistoryName = "databases";
    
    /*
     * Used to store the databases opened
     */
    //initDatabaseHistory();
}

/**
 * Destructor
 */
TeSQLite::~TeSQLite()
{
	if (bufferBlob_)
		delete bufferBlob_;
	if (isConnected_)
		close ();
}

/**
 * SQLite doesn't have a way to list the available databases, for this this
 * method initiates a special database for store the history of all opened
 * databases
 */
/*
bool TeSQLite::initDatabaseHistory()
{
    string dbPath = sqliteDir + dbHistoryName;
    
    // connect to 'databases'
    if ( sqlite3_open( dbPath.c_str(), &sqlite3_temp) != SQLITE_OK )
    {
        return false;
    }

    // create it if doesn't exists
    string query = "CREATE TABLE IF NOT EXISTS " + dbHistoryName + " (";
    query += "name TEXT NOT NULL, ";
	query += "path TEXT NOT NULL)";
    
    if ( sqlite3_exec(sqlite3_temp, query.c_str(), NULL, NULL, NULL) != SQLITE_OK )
    {
        sqlite3_close(sqlite3_temp);
        return false;
    }

    // closes the temporary connection
    sqlite3_close(sqlite3_temp);
    
    return true;
}
*/

/**
 * Register the opened databse to the list of availables one
 */
/*
bool TeSQLite::registerDatabase(const string db)
{
    string dbPath = sqliteDir + dbHistoryName;
    
    // connect temporary to 'databases'
    if ( sqlite3_open( dbPath.c_str(), &sqlite3_temp) != SQLITE_OK )
    {
        return false;
    }

    // check if the db is already registered
    string q = "SELECT name FROM " + dbHistoryName + " ";
    q += "WHERE name = '" + db + "' ";

    char **res;
    int *nrow;
    
    sqlite3_get_table(sqlite3_temp, q.c_str() , &res, nrow, NULL, NULL);
    
    // if not, register it
    if ( !nrow )
    {
        string query = "INSERT INTO " + dbHistoryName + " (";
        query += db + ", ";
        query += sqliteDir + db + ")";
        
        if ( sqlite3_exec(sqlite3_temp, query.c_str(), NULL, NULL, NULL) != SQLITE_OK )
        {
            sqlite3_close(sqlite3_temp);
            return false;
        }
    }

    // closes the temporary connection
    sqlite3_close(sqlite3_temp);
    
    return true;
}
*/

/**
 * Close the active connection
 */
void TeSQLite::close()
{
	clear();
	if (bufferBlob_)
    {
		delete bufferBlob_;
    }
	bufferBlob_ = 0;
	bufferBlobSize_ = 0;
	sqlite3_close(sqlite3_);
}

/**
 * Open or create a new database, if it does not exists
 */
bool TeSQLite::newDatabase(
        const string& database, 
        const string& user, 
        const string& password, 
        const string& host, 
        const int & /* port */, 
        bool terralibModel)
{
    
    string dbPath = sqliteDir + database;

    // connecting to 'database'
    // if 'database' does not exists it will be created
    if ( sqlite3_open( dbPath.c_str() , &sqlite3_) != SQLITE_OK )
    {
		isConnected_ = false;
		return false;
	}

    // register the opened database to the list of available databases
    //registerDatabase(database);

	if(terralibModel)
	{
		//create conceptual model
		if(!this->createConceptualModel())
        {
			return false;
        }
	}

	isConnected_ = true;
	user_ = user;
	host_ = host;
	password_ = password;
	database_ = database;
	return true;
}

/**
 * Open a database connection
 */
bool TeSQLite::connect (
        const string& host, 
        const string& user, 
        const string& password, 
        const string& database, 
        int port)
{

    string dbPath = sqliteDir + database;
    
    // connecting to 'database'
    if ( sqlite3_open( dbPath.c_str() , &sqlite3_) != SQLITE_OK )
    {
		isConnected_ = false;
		return false;
	}

    // register the opened database to the list of available databases
    //registerDatabase(database);
    
	isConnected_ = true;
	user_ = user;
	host_ = host;
	password_ = password;
	database_ = database;
	return true;
}

/**
 * Shows the list of available databases
 *
 * PS.: As the SQLite does not supports a way to list the available databases,
 * because its databases is files in the file system, this method uses the
 * auxiliar database that has a list of succesfull opened databases
 */
bool TeSQLite::showDatabases (
        const string& host, 
        const string& user, 
        const string& password, 
        vector<string>& dbNames, 
        int port)
{
    /*
    sqlite3 *sqlite3_temp;

    string dbPath = sqliteDir + dbHistoryName;
    
    // connect temporary to 'databases'
    if ( sqlite3_open( dbPath.c_str() , &sqlite3_temp) != SQLITE_OK )
    {
        return false;
    }

    char **res;
    int *nrow;
    int *ncolumn;

    // check if the db is already registered
    string q = "SELECT name FROM " + dbHistoryName + " ";
    
    sqlite3_get_table( sqlite3_temp, q.c_str(), &res, nrow, ncolumn,NULL);

    int i;
    for(i = *ncolumn; i < *nrow; i + *ncolumn)
    {
        dbNames.push_back(res[i]);
    } 
    
    // closes the temporary connection
    sqlite3_close(sqlite3_temp);
    */
	return true;
}


/**
 * Execute a SQL query
 */
bool TeSQLite::execute (const string &q)
{
    char *error;
    
	if ( sqlite3_exec(sqlite3_, q.c_str(), NULL, NULL, &error) != SQLITE_OK )
	{
        errorMessage_ = string(error);

        cout << " ERRO NESTA SQL: " << q << endl;
        
		return false;
	}

	return true;
}

string TeSQLite::errorMessage () 
{
    return string(sqlite3_errmsg(sqlite3_));
} 

int	TeSQLite::errorNum () 
{
    return sqlite3_errcode(sqlite3_);
}

/**
 * Check if a table exists
 */
bool TeSQLite::tableExist(const string& table)
{
	string q ="SELECT * FROM "+ table;
    char **result;
    char *error;
    
	if ( sqlite3_get_table(sqlite3_, q.c_str(), &result, NULL, NULL, &error) != SQLITE_OK )
	{
        errorMessage_ = string(error);
        return false;
	}
    else
    {
        return true;
    }
}

/**
 * Creates a table
 */
bool TeSQLite::createTable(const string& table, TeAttributeList &attr)
{
	string	q ="CREATE TABLE " + table +" (";
	string	type;
	bool	first=true;

	TeAttributeList::iterator it;
	it = attr.begin();
	while ( it != attr.end() )
	{
		string name = (*it).rep_.name_;
		
		switch ((*it).rep_.type_)
		{
			case TeSTRING:
				type = " TEXT ";
				break;
			case TeREAL:
				type = " REAL ";
				break;
			case TeINT:
				type = " INTEGER ";
				break;
			case TeCHARACTER:
				type = " TEXT ";
				break;
			case TeDATETIME:
				type = " TEXT ";  //time
				break;
			case TeBLOB:
				type = "BLOB ";
				break;
			default:
				type = " TEXT ";
				break;
		}

        if ( (*it).rep_.isPrimaryKey_ )
        {
            type += " PRIMARY KEY ";
        }
        
		if((*it).rep_.isAutoNumber_)
			type += " AUTOINCREMENT ";
				
		if (!first)
			q += ",";
		else
        {
			first = false;
            
        }
		
		q += name + " " + type;

		// check if column is part of primary key
		if ( (*it).rep_.isPrimaryKey_ && (*it).rep_.type_ != TeBLOB )
		{
			q += " NOT NULL ";
		}
		
		++it;
	}

	q += ")";

	return execute(q);
}


/**
 * Check if a column exists, and recover its attributes
 */
bool TeSQLite::columnExist(const string& table, const string&  column ,TeAttribute& attr)
{
    int res;
    const char* dataType;
    const char* collation;
    int* notNull;
    int* primaryKey;
    int* autoInc;
    
    res = sqlite3_table_column_metadata(
            sqlite3_, 
            NULL, 
            table.c_str(), 
            column.c_str(), 
            &dataType, 
            &collation, 
            notNull, 
            primaryKey, 
            autoInc
            );
    
    if ( res != SQLITE_OK )
    {
        return false;
    }

    attr.rep_.name_ = column;

    if ( strcmp( dataType, "TEXT" ) == 0 )
    {
		attr.rep_.type_ = TeSTRING;
    }
	else if ( strcmp( dataType, "INTEGER" ) == 0 )
    {
		attr.rep_.type_ = TeINT;
    }
	else if ( strcmp( dataType, "REAL" ) == 0 )
    {
		attr.rep_.type_ = TeREAL;
    }
	else if ( strcmp( dataType, "BLOB" ) == 0 )
    {
		attr.rep_.type_ = TeBLOB;
    }
	else if ( strcmp( dataType, "NULL" ) == 0 )
    {
		attr.rep_.type_ = TeUNKNOWN; // or NULL
    }
    
    // PS.:
    // As the SQLite does not set the size of its column fields, the attribute
    // attr.rep_.numChar_ is not filled for any
    
    if ( primaryKey )
    {
        attr.rep_.isPrimaryKey_ = true;
    }
	
    return true;
}

/**
 * Add a new column to the database
 */
bool TeSQLite::addColumn (const string& table, TeAttributeRep &rep)
{
	string q ="ALTER TABLE " + table +" ADD ";
	string type;
    
	string field = TeGetExtension(rep.name_.c_str());
	if(field.empty())
		field = rep.name_;

    /* As SQLite does not require datatypes for its columns, the definition
     * here just set the type affinity for the columns */
	switch (rep.type_)
	{
		case TeSTRING:
			type = " TEXT ";
			break;
		case TeREAL:
			type = " REAL ";
			break;
		case TeINT:
			type = " INTEGER ";
			break;
		case TeCHARACTER:
			type = " TEXT ";
			break;
		case TeDATETIME:
			type = " TEXT ";
			break;
		case TeBLOB:
			type = " BLOB ";
			break; 
		default:
    		type = " TEXT ";
    		break;
	}
	q += field;
	q += type;
	alterTableInfoInMemory(table);
	return execute(q);
}

/**
 * Definitions of TeDatabasePortal instance
 */
TeDatabasePortal* TeSQLite::getPortal ()
{
	TeSQLitePortal* portal = new TeSQLitePortal (this);
	return portal;
}


bool TeSQLite::createRasterTable (const string& table)
{
	if (table.empty())
		return false;

	string create = "CREATE TABLE "+table+" (";
	create += "block_id TEXT NOT NULL PRIMARY KEY, ";
	create += "lower_x REAL NOT NULL, ";
	create += "lower_y REAL NOT NULL, ";
	create += "upper_x REAL NOT NULL, ";
	create += "upper_y REAL NOT NULL, ";
	create += "band_id INTEGER NOT NULL, ";
	create += "resolution_factor INTEGER, ";
	create += "subband INTEGER,";
	create += "spatial_data BLOB NOT NULL, ";
	create += "block_size INTEGER NOT NULL)";

    string index = "CREATE INDEX " + table + "_index ON " + table + " ";
    index += "(band_id,lower_x,lower_y,upper_x,upper_y,resolution_factor,subband)";
    
    if ( !execute (create) )
    {
        return false;
    }

    if ( !execute (index) )
    {
        return false;
    }
    
	return true;
}


bool TeSQLite::createPolygonGeometry (const string&  table)
{
	if (table.empty())
		return false;

	string q ="CREATE TABLE " + table +"(";
	q += "geom_id INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,";
	q += "object_id TEXT NOT NULL ,";
	q += "num_coords INTEGER NOT NULL ,";
	q += "num_holes INTEGER NOT NULL ,";
	q += "parent_id INTEGER NOT NULL ,";
	q += "lower_x REAL NOT NULL ,";
	q += "lower_y REAL NOT NULL ,";
	q += "upper_x REAL NOT NULL ,";
	q += "upper_y REAL NOT NULL ,";
	q += "ext_max REAL NOT NULL ,";
	q += "spatial_data BLOB NOT NULL)";

    string index = "CREATE INDEX " + table + "_index ON " + table + " ";
    index += "(object_id,ext_max,lower_x,lower_y,upper_x,upper_y)";
    
    if ( !execute (q) )
    {
        return false;
    }

    if ( !execute (index) )
    {
        return false;
    }
    
	return true;
}


bool TeSQLite::createLineGeometry (const string& table)
{
	if (table.empty())
		return false;

	string q ="CREATE TABLE " + table +"(";
	q += "geom_id INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,";
	q += "object_id TEXT NOT NULL ,";
	q += "num_coords INTEGER not null ,";
	q += "lower_x REAL NOT NULL ,";
	q += "lower_y REAL NOT NULL ,";
	q += "upper_x REAL NOT NULL ,";
	q += "upper_y REAL NOT NULL ,";
	q += "ext_max REAL NOT NULL ,";
	q += "spatial_data BLOB NOT NULL)";

    string index = "CREATE INDEX " + table + "_index ON " + table + " ";
    index += "(object_id,lower_x,lower_y,upper_x,upper_y)";
    
    if ( !execute (q) )
    {
        return false;
    }

    if ( !execute (index) )
    {
        return false;
    }
    
	return true;
}


bool TeSQLite::defineIntegrity(void)
{
	return true;		// sqlite3 doesn't have referencial integrity
						// there is nothing to be done
}


TeSQLitePortal::TeSQLitePortal ( TeSQLite  *m)
{
	sqlite3_ = m->getSQLite();
	numRows_ = 0;
	numFields_ = 0;
	result_ = 0;
	row_.clear();
	db_ = m;
}


TeSQLitePortal::TeSQLitePortal ( const TeSQLitePortal& p) : TeDatabasePortal()
{
	sqlite3_ = p.sqlite3_;
	numRows_ = 0;
	numFields_ = 0;
	result_ = 0;
	row_.clear();
}


TeSQLitePortal::~TeSQLitePortal ()
{
	freeResult();
}

/**
 * Escape a string to a valid sql statement
 */
string TeSQLite::escapeSequence(const string& from)
{
	string ret = "";
	if (!from.empty())
	{
        char *to = sqlite3_mprintf("%q", from.c_str() );
		ret = string(to);
        sqlite3_free(to);
	}
	return ret;
}

/**
 * Execute a query and fill the fields attributes
 */
bool TeSQLitePortal::query (const string &q, TeCursorLocation /* l */, TeCursorType /* t */, TeCursorEditType /* e */, TeCursorDataType /*dt*/)
{
	if (result_)
    {
		freeResult();
    }
	attList_.clear ();
   
    // compiling the sql query
    if ( sqlite3_prepare(sqlite3_, q.c_str(), q.length(), &result_, NULL) != SQLITE_OK )
    {
        errorMessage_ = string(sqlite3_errmsg(sqlite3_));
        
        cout << " --->> ERRO na query: <<--- " << endl;
        
        cout << q << endl;
        
        cout << errorNum() << endl << endl;

        
		return false;
    }
    
    // check if it produces some results
	if (result_)  // there are rows
	{
        // obtaining the number of columns (fields) returned
        numFields_ = sqlite3_column_count(result_);

        // obtaining the number of rows and the attributes of each column
        int rows = 0;
        while( sqlite3_step(result_) == SQLITE_ROW )
        {
            rows++;

            if ( rows > 1 )
            {
                continue;
            }
            
            // looping on each column
            int i;
            for(i = 0; i < numFields_; i++)
    		{
                TeAttribute attribute;

                // obtaining the type of the current column
                switch ( sqlite3_column_type(result_, i) )
                {
                    case SQLITE_INTEGER: 
                        attribute.rep_.type_ = TeINT;
                        //attribute.rep_.numChar_ = (sqlite3_column_bytes( &result_, i) / sizeof(char));
                        break;
                    case SQLITE_FLOAT:
                        attribute.rep_.type_ = TeREAL;
                        //attribute.rep_.numChar_ = (sqlite3_column_bytes( &result_, i) / sizeof(char));
                        break;
                    case SQLITE_TEXT:
                        attribute.rep_.type_ = TeSTRING;
                        //attribute.rep_.numChar_ = (sqlite3_column_bytes( &result_, i) / sizeof(char));
                        break;
                    case SQLITE_BLOB:
                        attribute.rep_.type_ = TeBLOB;
                        break;
                    case SQLITE_NULL:
                        attribute.rep_.type_ = TeUNKNOWN;
                        break;
                    default :
        				attribute.rep_.type_ = TeUNKNOWN;
		        		break;
                }

                // obtaining the name of the current column
                char *column = (char*)sqlite3_column_name(result_,i);
                attribute.rep_.name_ = column;
               
                // obtaining the table name of the current column
                char *tablename = (char*)sqlite3_column_table_name(result_,i);

                // obtaining the database name of the current column
                char *dbname = (char*)sqlite3_column_database_name(result_,i);
                
                const char* dataType;
                const char* collation;
                int* notNull;
                int* primaryKey;
                int* autoInc;
                
                // obtaining meta data of the column
                sqlite3_table_column_metadata(sqlite3_, dbname, tablename, column, &dataType, &collation, notNull, primaryKey, autoInc);
                
                // obtaining the primary key information for this column
                if ( primaryKey )
				attribute.rep_.isPrimaryKey_ = true;

                // obtaining the auto increment information for this column
			    if ( autoInc )
				attribute.rep_.isAutoNumber_ = true;
                /////
                
                // push the atribute information
                attList_.push_back ( attribute );
            }
        }
        
        // number of rows
        numRows_ = rows;

        // reset the statement for the fetchrow method starts from begining
        sqlite3_reset(result_);
        
        // TODO:
        // check if is necessary to re-compile the statement after the reset
        // operation
        /*sqlite3_prepare(sqlite3_, q.c_str(), sizeof(q.c_str()), &result_, NULL);*/
        
        /*sqlite3_finalize(result_);*/
    }
    else  // returned nothing
    {
	    numFields_ = 0;
        numRows_ = 0;
    }
	return true;
}

/**
 * Fetch the next row of the result statement
 */
bool TeSQLitePortal::fetchRow ()
{
    if ( sqlite3_step(result_) == SQLITE_ROW )
    {
        int numF = sqlite3_column_count(result_);

        int i;
        for(i = 0; i < numF; i++)
        {
            row_.push_back((char*)sqlite3_column_text(result_, i));
        }
        
		return true;
    }
	return false;
}

/**
 * Fetch the 'i' row of the result statement
 */
bool TeSQLitePortal::fetchRow (int i)
{
    // reset the statement for the fetcharow method starts from begining
    sqlite3_reset(result_);

    // TODO:
    // check if is necessary to re-compile the statement after the reset
    // operation, running the 'query' method again
    
    // check if the 'i' row is valid
	if (i >= 0 && i < numRows_)
	{
        // executes the fetchrow 'i - 1' times to get exactly the desired row
        int j;
        for(j = 0; j < i; j++)
        {
            fetchRow();
        }

        // executes the exactly fetchrow for 'i'
        return fetchRow();
	}
    else
    {
	    return false;
    }
}


/**
 * Clean the result statement
 */
void TeSQLitePortal::freeResult ()
{
	if (result_)
	{
		sqlite3_finalize(result_);
		result_ = 0;
	}
}

/**
 * Returns the last error message
 */
string TeSQLitePortal::errorMessage () 
{
    return string(sqlite3_errmsg(sqlite3_));
}

/**
 * Returns the last error code
 */
int	TeSQLitePortal::errorNum () 
{
    return sqlite3_errcode(sqlite3_);
}

/**
 * Returns the last inserted ID
 */
int TeSQLitePortal::insertId ()
{
	return (int)sqlite3_last_insert_rowid(sqlite3_);
}

/**
 * Returns the last inserted ID
 */
int TeSQLite::insertId ()
{
	return (int)sqlite3_last_insert_rowid(sqlite3_);
}



bool TeSQLite::insertProjection (TeProjection *proj)
{
	char q[512];
	sprintf (q,"INSERT INTO te_projection VALUES(0,'%s',%.15f,%.15f,%.15f,%.15f,%.15f,%.15f,'%s',%.15f,%d,'%s',%.15f,%.15f,%.15f,%.15f,%.15f)",
	proj->name().c_str(),
		proj->params().lon0*TeCRD,
		proj->params().lat0*TeCRD,
		proj->params().offx,
		proj->params().offy,
		proj->params().stlat1*TeCRD,
		proj->params().stlat2*TeCRD,
		proj->params().units.c_str(),
		proj->params().scale,
		proj->params().hemisphere,
		proj->datum().name().c_str(),
		proj->datum().radius(),
		proj->datum().flattening(),
		proj->datum().xShift(),
		proj->datum().yShift(),
		proj->datum().zShift());

	if (!execute(q))
		return false;

	proj->id(insertId());
	return true;
}


bool TeSQLite::insertView (TeView *view)
{
	TeProjection* proj = view->projection();
	// save its projection
	if (!proj || !insertProjection(proj))
	{
		errorMessage_ = "Nao e possivel inserir vista sem projeto";
		return false;
	}

	char q[512];
	sprintf (q,"INSERT INTO te_view VALUES(0,%d,'%s','%s',%d)",
			proj->id(),view->name().c_str(),view->user().c_str(),view->isVisible());

	if (!execute(q))
		return false;

	view->id(insertId());

	for (unsigned int th=0; th<view->size(); th++)
	{
		TeViewNode* node = view->get (th);
		if (node->type() == TeTHEME)
		{
			TeTheme *theme = (TeTheme*) node;
			insertTheme (theme);
		}
		else
		{
			TeViewTree* tree = (TeViewTree*)node;
			insertViewTree (tree);
		}
	}

	viewMap_[view->id()] = view;
	return true;
}


bool TeSQLite::insertViewTree (TeViewTree *tree)
{
	string save;
	save = "INSERT INTO te_theme (name, layer_id, node_type, view_id, priority) VALUES('','";
	save += tree->name() +  "," + Te2String(tree->parentId()) + "," + Te2String((int)tree->type());
	save += "," + Te2String(tree->view()) + "," + Te2String(tree->priority()) + ")";
	if (!execute(save))
		return false;
	tree->id(insertId());
	return true;
}


bool TeSQLite::insertTheme (TeTheme *tem)
{
	string save;
	save = "INSERT INTO te_theme ";
	save += "(layer_id, view_id, name, parent_id, priority, node_type, min_scale, max_scale, ";
	save += "generate_attribute_where, generate_spatial_where, generate_temporal_where, ";
	save += "collection_table, visible_rep, enable_visibility) ";
	save += " VALUES(";
	save += Te2String(tem->layerId()) +", ";
	save += Te2String(tem->view()) +", ";
	save += "'" + tem->name() + "', ";
	save += Te2String(tem->parentId()) +", ";
	save += Te2String(tem->priority()) +", ";
	save += Te2String(tem->type()) +", ";
	save += Te2String(tem->minScale()) +", ";
	save += Te2String(tem->maxScale()) +", ";
	save += "'" + escapeSequence(tem->attributeRest()) + "', ";
	save += "'" + escapeSequence(tem->spatialRest()) + "', ";
	save += "'" + escapeSequence(tem->temporalRest()) + "', ";
	save += "'" + escapeSequence(tem->collectionTable()) + "', ";
	save += Te2String(tem->visibleRep()) +", ";
	save += Te2String(tem->visibility()) +" ) ";

	if (!this->execute(save))
		return false;

	tem->id(this->insertId());

	if (tem->collectionTable().empty())
	{
		string colTab = "te_collection_" + Te2String(tem->id());
		tem->collectionTable(colTab);
		save = "UPDATE te_theme SET collection_table='" + colTab + "' WHERE theme_id=" ;
		save += Te2String(tem->id());
		this->execute(save);
	}

	bool status;

	// insert grouping
	int numSlices = 0;
	
	TeGrouping* tmpGrouping = NULL;
	tmpGrouping = new TeGrouping(tem->grouping());
	
	if(tmpGrouping)
	{
		if(!insertGrouping (tem->id(), *tmpGrouping))
			return false;
		numSlices = tem->grouping().groupNumSlices_;
	}
		
	tem->outOfCollectionLegend().group(-1); 
	tem->outOfCollectionLegend().theme(tem->id()); 
	status = insertLegend (&(tem->outOfCollectionLegend())); 
	if (!status)
		return status;

	tem->withoutDataConnectionLegend().group(-2); 
	tem->withoutDataConnectionLegend().theme(tem->id()); 
	status = insertLegend (&(tem->withoutDataConnectionLegend())); 
	if (!status)
		return status;

	tem->defaultLegend().group(-3); 
	tem->defaultLegend().theme(tem->id()); 
	status = insertLegend (&(tem->defaultLegend())); 
	if (!status)
		return status;

	tem->pointingLegend().group(-4); 
	tem->pointingLegend().theme(tem->id()); 
	status = insertLegend (&(tem->pointingLegend())); 
	if (!status)
		return status;

	tem->queryLegend().group(-5); 
	tem->queryLegend().theme(tem->id()); 
	status = insertLegend (&(tem->queryLegend())); 
	if (!status)
		return status;

	tem->queryAndPointingLegend().group(-6); 
	tem->queryAndPointingLegend().theme(tem->id()); 
	status = insertLegend (&(tem->queryAndPointingLegend())); 
	if (!status)
		return status;

	for (int i = 0; i < numSlices; i++)
	{
		tem->legend()[i].group(i);
		tem->legend()[i].theme(tem->id());
		status = insertLegend (&(tem->legend()[i]));
		if (!status)
			return status;
	}
	if (!status)
		return status;

	themeMap_[tem->id()] = tem;
	return updateThemeTable(tem);
}

bool TeSQLite::insertThemeGroup (TeViewTree* tree)
{
	string save;
	save = "INSERT INTO te_theme ";
	save += "(name,parent_id,node_type,view_id,priority) ";
	save += " VALUES ('" + tree->name() + "',1,1, ";
	save += Te2String(tree->view()) +", ";
	save += Te2String(tree->priority()) +") ";
	if (!this->execute(save))
		return false;
	tree->id(this->insertId());
	return true;
}


bool TeSQLite::generateLabelPositions (TeTheme *theme)
{
	string	geomTable, query;
	string	collTable = theme->collectionTable();

	if((collTable.empty()) || (!tableExist(collTable)))
		return false;

	if (theme->layer()->hasGeometry(TeLINES))
	{	
		geomTable = theme->layer()->tableName(TeLINES);
				
		string extMax = "CREATE TEMPORARY TABLE maxTable";
		extMax += " SELECT object_id, lower_x, lower_y, upper_x, upper_y";
		extMax += " FROM " + geomTable + " ORDER BY object_id, ext_max DESC";
		
		if(!execute(extMax))
			return false;

		query = "UPDATE " + collTable + ",maxTable SET label_x = (lower_x + upper_x)/2,";
		query += "label_y = (lower_y + upper_y)/2 WHERE c_object_id = object_id";

		if (execute(query) == false)
		{
			execute("DROP TABLE maxTable");
			return false;
		}

		execute("DROP TABLE maxTable");
	}

	if (theme->layer()->hasGeometry(TePOINTS))
	{
		geomTable = theme->layer()->tableName(TePOINTS);

		query = "UPDATE " + collTable + "," + geomTable + " SET label_x = x,";
		query += "label_y = y WHERE c_object_id = object_id";

		if(execute(query) == false)
			return false;
	}
	
	if (theme->layer()->hasGeometry(TePOLYGONS))
	{	
		geomTable = theme->layer()->tableName(TePOLYGONS);
				
		string extMax = "CREATE TEMPORARY TABLE maxTable";
		extMax += " SELECT object_id, lower_x, lower_y, upper_x, upper_y";
		extMax += " FROM " + geomTable + " ORDER BY object_id, ext_max DESC";
		
		if(!execute(extMax))
			return false;

		query = "UPDATE " + collTable + ",maxTable SET label_x = (lower_x + upper_x)/2,";
		query += "label_y = (lower_y + upper_y)/2 WHERE c_object_id = object_id";

		if (execute(query) == false)
		{
			execute("DROP TABLE maxTable");
			return false;
		}
		
		execute("DROP TABLE maxTable");
	}
	
	if(theme->layer()->hasGeometry(TeCELLS))
	{
		geomTable = theme->layer()->tableName(TeCELLS);

		query = "UPDATE " + collTable + "," + geomTable + " SET label_x = (lower_x + upper_x)/2,";
		query += "label_y = (lower_y + upper_y)/2 WHERE c_object_id = object_id";

		if (execute(query) == false)
			return false;	
	}
	
	return true;
}


bool TeSQLite::insertLayer (TeLayer* layer)
{
	cout << "instertLayer" << endl;
	TeProjection* proj = layer->projection();
	if (!proj || !insertProjection(proj))
	{
		errorMessage_ = "Layer sem proje�o ou falha na inser�o da proje�o";
		return false;
	}
	int pid = proj->id();
	string save = "INSERT INTO te_layer (layer_id,projection_id,name,lower_x,lower_y,upper_x,upper_y) VALUES(0,";
	save += Te2String(pid);
	save += ",'" + layer->name() + "', ";
	save += Te2String(layer->box().x1_,15)+ ", ";
	save += Te2String(layer->box().y1_, 15)+ ", ";
	save += Te2String(layer->box().x2_, 15)+ ", ";
	save += Te2String(layer->box().y2_, 15) + ")";

	// if layer insertion fails remove projection
	if (!this->execute (save))
	{
		string sql = "DELETE FROM te_projection WHERE  projection_id = " + Te2String(pid);
		this->execute(sql);	
		cout << "instertLayer return false" << endl;
		return false;
	}

	layer->id(this->insertId());
	layerMap_[layer->id()] = layer;

	cout << "instertLayer return true" << endl;
	return true;
}


bool TeSQLite::insertLegend (TeLegendEntry *legend)
{
	char q[1024];
	string save;
	sprintf (q,"INSERT INTO te_legend VALUES(0,%d,%d,%d,'%s','%s','%s')",
	legend->theme(),
	legend->group(),
	legend->count(),
	legend->from().c_str(),
	legend->to().c_str(),
	legend->label().c_str());
	save = q;

	if (!this->execute(save))
		return false;

	int legId = insertId();
	legend->id(legId);
	legendMap_[legend->id()] = legend;
	return insertVisual(legend);
}


bool TeSQLite::insertRepresentation (int layerId, TeRepresentation& rep)
{
	if (layerId <= 0)
		return false;
	
	string sql;
	sql = "INSERT INTO te_representation (layer_id, geom_type, geom_table, description,";
	sql += "lower_x, lower_y, upper_x, upper_y, res_x, res_y, num_cols, num_rows) VALUES( ";
	sql += Te2String(layerId) + ", ";
	sql += Te2String(static_cast<int>(rep.geomRep_)) + ", ";
	sql += "'" + rep.tableName_ + "', ";
	sql += "'" + rep.description_ + "',";
	sql += Te2String(rep.box_.x1_) + ", ";
	sql += Te2String(rep.box_.y1_) + ", ";
	sql += Te2String(rep.box_.x2_) + ", ";
	sql += Te2String(rep.box_.y2_) + ", ";
	sql += Te2String(rep.resX_) + ", ";
	sql += Te2String(rep.resY_) + ", ";
	sql += Te2String(rep.nCols_) + ", ";
	sql += Te2String(rep.nLins_) + ") ";
	if (execute(sql))
	{
		rep.id_ = (this->insertId());
		return true;
	}
	return false;
}

bool TeSQLite::insertRelationInfo(const int tableId, const string& tField,
						    const string& eTable, const string& eField, int& relId)
{
	// check if relation already exists
	TeSQLitePortal* portal = (TeSQLitePortal*) this->getPortal();
	if (!portal)
		return false;

	relId = -1;
	string sel = "SELECT relation_id FROM te_tables_relation WHERE";
	sel += " related_table_id = " + Te2String(tableId);
	sel += " AND related_attr = '" + tField + "'";
	sel += " AND external_table_name = '" + eTable + "'";
	sel += " AND external_attr = '" + eField + "'";
	if (!portal->query(sel))
	{
		delete portal;
		return false;
	}

	if (portal->fetchRow())
	{
		relId = atoi(portal->getData(0));
		delete portal;
		return true;
	}
	delete portal;

	string sql = "INSERT INTO te_tables_relation(related_table_id, related_attr, external_table_name, external_attr) ";
	sql += " VALUES( ";
	sql += Te2String(tableId);
	sql += ",'" + tField + "'";
	sql += ",'" + eTable + "'";
	sql += ",'" + eField + "')";
	if (!this->execute(sql))
		return false;
	relId = this->insertId(); 
	return true;
}

bool TeSQLite::insertTableInfo (int layerId, TeTable &table, const string& user)
{
	string sql;

	sql = "INSERT INTO te_layer_table (unique_id, attr_link,attr_initial_time, attr_final_time, attr_time_unit,attr_table_type, ";
	sql += " user_name, attr_table ";
	if (layerId > 0)
		sql += ",layer_id";
	sql += ") VALUES( ";
	sql += "'" + table.uniqueName() + "', ";
	sql += "'" + table.linkName() + "', ";
	sql += "'" + table.attInitialTime() + "', ";
	sql += "'" + table.attFinalTime() + "', ";
	sql += "" + Te2String(table.attTimeUnit()) + ", ";
	sql += ""+ Te2String(table.tableType()) + ", ";
	sql += "'" + user + "','" + table.name() +"'";
	if (layerId > 0)
		sql += ", " + Te2String(layerId) ;
	sql += ")";

	if (!this->execute(sql))
		return false;

	int id = (this->insertId());
	table.setId(id);
	return true;
}

bool TeSQLite::insertThemeTable (int themeId, int tableId, int relationId, int tableOrder)
{
	string relId;
	if (relationId > 0)
		relId = Te2String(relationId);
	else
		relId = "NULL";

	string sql = "INSERT INTO te_theme_table VALUES (0,";
	sql += Te2String(themeId) + "," + Te2String(tableId) + ",";
	sql += relId + "," + Te2String(tableOrder) + ")";


	if (!execute(sql))
		return false;

	return true;
}


bool TeSQLite::insertPolygon (const string& table, TePolygon &poly )
{
	double	ext_max;

	int	i,
		ni,
		size;
	unsigned int k;
	double	*points	= NULL;
	char	buf[256], *query, *end;


	int parentId = 0;
	for ( k = 0; k < poly.size(); k++ )
	{
		TeLinearRing ring ( poly[k] );
		TeBox b = ring.box();
		size = ring.size();
		ni = 0;
		ext_max = MAX(b.width(),b.height());
		if (k==0)
			ni = poly.size()-1;
		else
			ni = 0;

		// Create and save the BLOB
		points = new double[2*sizeof(double)*size];

		int iac = 0;
		for (i=0;i<size;i++)
		{
			points[iac++]=swappd2(ring[i].x());
			points[iac++]=swappd2(ring[i].y());
		}

		sprintf (buf,"INSERT INTO %s values(0,'%s',%d,%d,%d,%.15f,%.15f,%.15f,%.15f,%.15f,",
			table.c_str(),escapeSequence(poly.objectId()).c_str(),size,ni,parentId,b.lowerLeft().x(),b.lowerLeft().y(),b.upperRight().x(),b.upperRight().y(),ext_max);

		query = new char[size*4*sizeof(double)+strlen(buf)+4];
		end = strxmov(query,buf,(char*)0);
		*end++ = '\'';
        
        // TODO: Check if this concat operation is right
        string escaped = escapeSequence(string((char*)points));
		end = strcat(end,escaped.c_str());
		
        *end++ = '\'';
		*end++ = ')';
		delete points;

		if ( !execute(string(query)) )
		{
			errorMessage_ = string(sqlite3_errmsg(sqlite3_));
			delete query;
			return false;
		}
		delete query;
		query = 0;
		ring.geomId (insertId());
		if (k==0)
			parentId = ring.geomId();
	}
	sprintf(buf,"UPDATE %s SET parent_id=%d WHERE geom_id=%d",table.c_str(),parentId,parentId);
	if ( !execute(string(buf)) )
	{
		errorMessage_ = string(sqlite3_errmsg(sqlite3_));
		delete query;
		return false;
	}	
	return true;
}

bool TeSQLite::updatePolygon (const string& table, TePolygon &poly )
{
	double	ext_max;
	int ni;
	unsigned int i,k,size;
	double	*points	= NULL;
	char	buf[256], message[256], *query, *end;
	TeBox b = poly.box();

	for ( k = 0; k < poly.size(); k++ )
	{
		TeLinearRing ring ( poly[k] );
		size = ring.size();
		ni = 0;
		if (k==0)
		{
			ext_max = MAX(b.width(),b.height());
			ni = poly.size()-1;
		}
		else
			ni = 0;

// Create and save the BLOB
		points = new double[2*sizeof(double)*size];
		int iac = 0;
		for (i=0;i<size;i++)
		{
			points[iac++]=swappd2(ring[i].x());
			points[iac++]=swappd2(ring[i].y());
		}

		sprintf (buf,"UPDATE %s SET num_coords=%d, num_holes=%d, parent_id=%d, lower_x=%g, lower_y=%g, upper_x=%g, upper_y=%g, ext_max=%g, spatial_data=",
			table.c_str(),size,ni,k,b.lowerLeft().x(),b.lowerLeft().y(),b.upperRight().x(),b.upperRight().y(),ext_max);
		query = new char[size*4*sizeof(double)+strlen(buf)+50];
		end = strxmov(query,buf,(char*)0);
		*end++ = '\'';

        // TODO: Check if this concat operation is right
        string escaped = escapeSequence(string((char*)points));
		end = strcat(end,escaped.c_str());
        
		*end++ = '\'';
		sprintf (message," WHERE geom_id=%d",ring.geomId());
		end = strxmov(end,message,0);
		delete points;

		if ( !execute(string(query)) )
		{
			errorMessage_ = string(sqlite3_errmsg(sqlite3_));
			delete query;
			return false;
		}
		delete query;
		query = 0;
	}
	return true;
}


bool TeSQLitePortal::fetchGeometry (TePolygon& poly)
{
	int ni,j;
	TeLinearRing ring = this->getLinearRing(ni);
	poly.objectId ( ring.objectId() );
	poly.geomId ( ring.geomId() );
	poly.add ( ring );
	int parentId = poly.geomId();
	while (fetchRow())
	{
		if (atoi(this->getData("parent_id")) == parentId)
		{
			TeLinearRing ring = getLinearRing(j);
			poly.add ( ring );
		}
		else
			return true;
	}
	return false;
}


TeLinearRing TeSQLitePortal::getLinearRing (int &ni)
{
	int index = atoi(getData("geom_id"));
	string object_id = getData("object_id");
	int np = atoi (getData("num_coords"));
	ni = atoi (getData("num_holes"));
	TeBox b (getDouble("lower_x"),getDouble("lower_y"),getDouble("upper_x"),getDouble("upper_y"));
	int k;
	TeLine2D line;
	double x,y,*data = (double*)getData("spatial_data");

	for (k=0;k<np;k++)
	{
		x = swappd2(*data++);
		y = swappd2(*data++);
		TeCoord2D pt(x,y);
		line.add(pt);
	}

	TeLinearRing ring ( line );
	ring.objectId (object_id);
	ring.geomId (index);
	ring.setBox (b);
	return ring;
}


bool TeSQLite::insertLine (const string& table, TeLine2D &l)
{
	char	buf[256],
			*query,
			*end;
	double	*points	= NULL,
			ext_max;
	int		size=l.size(),
			i,
			iac;
	TeBox	b=l.box();

	ext_max = MAX(b.width(),b.height());
	points = new double[2*sizeof(double)*size];
	iac = 0;

	for (i=0;i<size;i++)
	{
		points[iac++]=l[i].x();
		points[iac++]=l[i].y();
	}

	sprintf (buf,"INSERT INTO %s values(0,'%s',%d,%.15f,%.15f,%.15f,%.15f,%.15f,",
	table.c_str(),
	escapeSequence(l.objectId()).c_str(),
	size,
	b.lowerLeft().x(),b.lowerLeft().y(),b.upperRight().x(),b.upperRight().y(),ext_max);

	query = new char[size*4*sizeof(double)+strlen(buf)+4];

	end = strxmov(query,buf,0);
	*end++ = '\'';
    
    // TODO: Check if this concat operation is right
    string escaped = escapeSequence(string((char*)points));
	end = strcat(end,escaped.c_str());
        
	*end++ = '\'';
	*end++ = ')';
	delete points;

	if ( !execute(string(query)) )
	{
		errorMessage_ = string(sqlite3_errmsg(sqlite3_));
		delete query;
		return false;
	}

	l.geomId (insertId());
	delete query;
	return true;
}


bool TeSQLite::updateLine (const string& table, TeLine2D &l)
{
	char	buf[256],
			message[256],
			*query,
			*end;
	double	*points	= NULL,
			ext_max;
	int		size=l.size(),
			i,
			iac;
	TeBox	b=l.box();

	ext_max = MAX(b.width(),b.height());
	points = new double[2*sizeof(double)*size];
	iac = 0;

	for (i=0;i<size;i++)
	{
		points[iac++]=l[i].x();
		points[iac++]=l[i].y();
	}

	sprintf (buf,"UPDATE %s SET num_coords=%d, lower_x=%g, lower_y=%g, upper_x=%g, upper_y=%g, ext_max=%g, spatial_data=",
	table.c_str(),size,b.lowerLeft().x(),b.lowerLeft().y(),b.upperRight().x(),b.upperRight().y(),ext_max);
	query = new char[size*4*sizeof(double)+strlen(buf)+50];
	end = strxmov(query,buf,(char*)0);
	*end++ = '\'';
    
    // TODO: Check if this concat operation is right
    string escaped = escapeSequence(string((char*)points));
	end = strcat(end,escaped.c_str());
	
    *end++ = '\'';
	delete points;

	sprintf (message," WHERE geom_id=%d",l.geomId());
	end = strxmov(end,message,(char*)0);

	if ( !execute(string(query)) )
	{
		errorMessage_ = string(sqlite3_errmsg(sqlite3_));
		delete query;
		return false;
	}
	delete query;
	return true;
}



bool TeSQLitePortal::fetchGeometry(TeLine2D& line)
{
	int index = atoi (getData ("geom_id"));
	string object_id = getData ("object_id");
	int np = atoi (getData("num_coords"));
	TeBox b (atof(getData("lower_x")),atof(getData("lower_y")),atof(getData("upper_x")),atof(getData("upper_y")));
	int k;
	double x,y,*data = (double*)getData("spatial_data");

	for (k=0;k<np;k++)
	{
		x = *data++;
		y = *data++;
		TeCoord2D pt(x,y);
		line.add(pt);
	}
	line.objectId (object_id);
	line.geomId (index);
	line.setBox (b);
	return (fetchRow());
}


bool TeSQLite::insertPoint(const string& table,TePoint &p)
{
	string q = "INSERT INTO " + table + " VALUES(0,";
	q += "'" + escapeSequence(p.objectId()) + "',";
	q += Te2String(p.location().x_) + ",";
	q += Te2String(p.location().y_) + ")";
	
	if (!this->execute(string(q)))
		return false;
	p.geomId(this->insertId());
	return true;
}


bool TeSQLitePortal::fetchGeometry(TePoint& p)
{
	TeCoord2D c(atof (getData("x")), atof (getData("y")));
	p.geomId( atoi(getData("geom_id")));
	p.objectId( string(getData("object_id")));
	p.add(c);
	return (fetchRow());
}


bool TeSQLite::insertText(const string& table, TeText &t)
{
	char q[256];
	sprintf (q,"INSERT INTO %s values(0,'%s',%g,%g,'%s',%g, %g, %g, %g)",
		table.c_str(), escapeSequence(t.objectId()).c_str(),
		t.location().x_, t.location().y_,
		t.textValue().c_str(), t.angle(), t.height(),t.alignmentVert(),t.alignmentHoriz());
	if (!this->execute(string(q)))
		return false;
	t.geomId(this->insertId());

	return true;
}


bool TeSQLite::insertArc(const string& table, TeArc &arc)
{
	char q[256];
	sprintf (q,"INSERT INTO %s values(0,'%s',%d,%d)",
		table.c_str(),escapeSequence(arc.objectId()).c_str(),arc.fromNode().geomId(),arc.toNode().geomId());
	if (!this->execute(string(q)))
		return false;
	arc.geomId(this->insertId());
	return true;
}


bool TeSQLite::insertNode(const string& table, TeNode &node)
{
	char q[256];
	sprintf (q,"INSERT INTO %s values(0,'%s',%g,%g)",table.c_str(),node.objectId().c_str(),node.location().x(),node.location().y());
	if (!this->execute(string(q)))
		return false;
	node.geomId(this->insertId());
	return true;
}

bool TeSQLite::insertBlob (const string& tableName, const string& columnBlob , const string& whereClause, unsigned char* data, int size)
{
	if (whereClause.empty())
		return false;

	TeSQLitePortal* portal = (TeSQLitePortal*)this->getPortal();
	if (!portal)
		return false;

	string q = "SELECT * FROM "+ tableName +" WHERE "+ whereClause;
	if ((!portal->query(q)) || (!portal->fetchRow()))
	{
		delete portal;
		return false;
	}
	delete portal;

	char	message[256],
			*query,
			*end;
	sprintf (message,"UPDATE %s SET %s = ",tableName.c_str(), columnBlob.c_str());

	query = new char[2*size+strlen(message)+4];
	end = strxmov(query,message,0);
	*end++ = '\'';

    // TODO: Check if this concat operation is right
    string escaped = escapeSequence(string((char*)data));
	end = strcat(end,escaped.c_str());
    
	*end++ = '\'';

	sprintf (message," WHERE %s",whereClause.c_str());

	end = strxmov(end,message,0);

	if ( !execute(string(query)) )
	{
		errorMessage_ = string(sqlite3_errmsg(sqlite3_));
		delete query;
		return false;
	}
	delete query;	
	return true;
}

bool TeSQLite::insertBlob (const string& tableName, const string& columnBlob, const string& whereClause, const string& fileName)
{
	if (whereClause.empty())
		return false;

	unsigned char	*cdata = 0;
	int		size;
	FILE	*fp = 0;
	
	struct	stat buf;
	int		result;
	
	result = stat(fileName.c_str(), &buf);
	
	if( result != 0 )
		return false;
	
	size = buf.st_size;

	cdata = new unsigned char[size];
	fp = fopen(fileName.c_str(), "rb");
	fread(cdata, sizeof(unsigned char), size, fp); 

	bool status = insertBlob (tableName, columnBlob, whereClause, cdata, size);

	if(fp)
		fclose(fp);

	if (cdata)
		delete []cdata;

	return status;
}

bool TeSQLite::insertBlob (const string& tableName, const string& columnBlob, TeAttributeRep& columnId, const string& valueId, const string& fileName)
{
	string whereClause = columnId.name_ +" = ";
	switch (columnId.type_ )
	{
		case TeSTRING:
			whereClause += "'"+ valueId + "'";
			break;
		default:
			whereClause += valueId;
			break;
	}
	return (insertBlob(tableName,columnBlob,whereClause,fileName));
}

bool TeSQLite::insertBlob (const string& tableName, const string& columnBlob , TeAttributeRep& columnId, const string& valueId, unsigned char* data, int size)
{
	string whereClause = columnId.name_ +" = ";
	switch (columnId.type_ )
	{
		case TeSTRING:
			whereClause += "'"+ valueId + "'";
			break;
		default:
			whereClause += valueId;
			break;
	}
	return (insertBlob(tableName,columnBlob,whereClause,data,size));
}
	
bool TeSQLite::insertRasterBlock(const string& table, const string& blockId, const TeCoord2D& ll, const TeCoord2D& ur, unsigned char *buf,unsigned long size, int band, unsigned int res, unsigned int subband)
{
	if (blockId.empty()) // no block identifier provided
	{
		errorMessage_ = "bloco sem identificador";
		return false;
	}

	TeSQLitePortal* portal = (TeSQLitePortal*) this->getPortal();
	if (!portal)
		return false;

	bool update = false;
	string q ="SELECT * FROM " + table; 
	q += " WHERE block_id='" + blockId + "'";

	if (!portal->query (q))
	{
		delete portal;
		return false;
	}
		// check if this block is alread in the database
	if (portal->fetchRow())
		update = true;
	delete portal;

	char	message[256],
			*end;
	if (!update)
	{
		sprintf (message,"INSERT INTO %s values('%s',%f,%f,%f,%f,%d,%d,%d,",
			table.c_str(),blockId.c_str(),ll.x(),ll.y(),ur.x(),ur.y(),band,res,subband);
	}
	else
	{
		sprintf (message,"UPDATE %s SET spatial_data=",table.c_str());
	}

	if (bufferBlobSize_ < (long)(2*size+strlen(message)+20))
	{
		if (bufferBlob_)
			delete bufferBlob_;
		bufferBlobSize_ = (2*size+strlen(message)+20);
		bufferBlob_ = new char[bufferBlobSize_];
	}
	end = strxmov(bufferBlob_,message,0);
	*end++ = '\'';

    // TODO: Check if this concat operation is right
    string escaped = escapeSequence(string((char*)buf));
	end = strcat(end,escaped.c_str());
    
	*end++ = '\'';

	if (update)
	{
		sprintf (message,", block_size=%ld WHERE block_id='%s'",size,blockId.c_str());
		end = strxmov(end,message,0);
	}
	else
	{
		sprintf (message,",%ld)",size);
		end = strxmov(end,message,0);
	}

	if ( !execute(string(bufferBlob_)) )
	{
		errorMessage_ = string(sqlite3_errmsg(sqlite3_));
		delete bufferBlob_;
		bufferBlobSize_ = 0;
		bufferBlob_ = 0;
		return false;
	}
	return true;
}

bool TeSQLitePortal::fetchGeometry(TeNode& n)
{
	TeCoord2D c(atof (getData("x")), atof (getData("y")));
	n.geomId( atoi(getData("geom_id")));
	n.objectId( string(getData("object_id")));
	n.add(c);
	return (fetchRow());
}


bool TeSQLite::insertCell(const string& table, TeCell &c)
{
	TeBox	b=c.box();
	char	q[256];
	sprintf (q,"INSERT INTO %s values(0,'%s',%.15f,%.15f,%.15f,%.15f,%d,%d)",
		table.c_str(),c.objectId().c_str(),
		b.lowerLeft().x(),b.lowerLeft().y(),b.upperRight().x(),b.upperRight().y(),
		c.column(),c.line());

	if (!this->execute (string(q)))
		return false;
	c.geomId(this->insertId());
	return true;
}


TeTime TeSQLitePortal::getDate (int i)
{ 
	string s = getData(i);
	TeTime t(s, TeSECOND, "YYYYsMMsDDsHHsmmsSS","-"); 
	return t; 
}

TeTime TeSQLitePortal::getDate (const string& s)
{ 
	string sv = getData(s);
	TeTime t(sv, TeSECOND, "YYYYsMMsDDsHHsmmsSS","-"); 
	return t; 
}

string TeSQLitePortal::getDateAsString(const string& s)
{
	TeTime t = this->getDate(s);
	if (t.isValid())
	{		
		string tval = "\'"+t.getDateTime("YYYYsMMsDDsHHsmmsSS", "-")+"\'";
		return tval;
	}
	else
		return "";
}

string TeSQLitePortal::getDateAsString(int i)
{
	TeTime t = this->getDate(i);
	if (t.isValid())
	{		
		string tval = "\'"+t.getDateTime("YYYYsMMsDDsHHsmmsSS", "-")+"\'";
		return tval;
	}
	else
		return "";
}

double TeSQLitePortal::getDouble (const string& s)
{
	return atof(getData(s));
}


double TeSQLitePortal::getDouble (int i)
{
	if (row_.empty())
		return 0.;

    string a = "";
    a = row_.at(i);
    
	if ( a.c_str() )
    {
        string str = row_.at(i);
		return atof(str.c_str());
    }
    else
    {
		return 0.;
    }
}

int TeSQLitePortal::getInt (const string& s)
{
	return atoi(getData(s));
}

int TeSQLitePortal::getInt (int i)
{
	if (row_.empty())
		return 0;

    string a = "";
    a = row_.at(i);

	if ( a.c_str() )
    {
        string str = row_.at(i);
		return atoi(str.c_str());
    }
	else
    {
		return 0;
    }
}


char* TeSQLitePortal::getData (int i)
{
	if (row_.empty())
		return "";

    string data = "";
    data = row_.at(i);
    
	if ( data.c_str() )
    {
		return (char*)data.c_str();
    }
    else
    {
		return "";
    }
}


char* TeSQLitePortal::getData (const string& s)
{
	string fieldName;
//	size_t pos = s.find(".", string::npos,1);
	size_t pos = s.find(".", 0, 1);
	if (pos != string::npos)
		fieldName = s.substr(pos+1);
	else
		fieldName = s;

	if (row_.empty())
		return "";

	TeAttributeList::iterator it = attList_.begin();

	int j = 0;
	while ( it != attList_.end() )
	{
		if ((TeConvertToUpperCase(fieldName) == TeConvertToUpperCase((*it).rep_.name_)) ||
		    (TeConvertToUpperCase(s) == TeConvertToUpperCase((*it).rep_.name_)))
		{
            string data = "";
            data = row_.at(j);
            
			if ( data.c_str() )
				return (char*)data.c_str();
			else
				return "";
		}
		++it;
		j++;
	}
	return "";
}

bool TeSQLitePortal::getBlob (const string& s, unsigned char* &data, long& size)
{
	string fieldName;
//	size_t pos = s.find(".", string::npos,1);
	size_t pos = s.find(".", 0, 1);
	if (pos != string::npos)
		fieldName = s.substr(pos+1);
	else
		fieldName = s;

	TeAttributeList::iterator it = attList_.begin();
	int j = 0, i=-1;
	while ( it != attList_.end() )
	{
		if (fieldName == (*it).rep_.name_)
		{
			i=j;
			break;
		}
		++it;
		j++;
	}
	if (i<0)
		return false;

    /*
	unsigned long *lengths;
	lengths = mysql_fetch_lengths(result_);
    */
	size = sqlite3_column_bytes(result_,i);
    
    string a = "";
    a = row_.at(i);
    
	if ( a.c_str() )
	{
//		data = (unsigned char*) row_[i];
		data = new unsigned char[size];
		memcpy(data, (unsigned char*) a.c_str(), size * sizeof(char));
		return true;
	}
	else
	{
		data = 0;
		return false;
	}
}


bool TeSQLitePortal::getBool (const string& s)
{
	int val = atoi(getData(s));
	return val ? true : false;
}


bool TeSQLitePortal::getBool (int i)
{
	if (row_.empty())
		return false;
	int val = atoi(getData(i));
	return val ? true : false;
}

bool TeSQLitePortal::getRasterBlock(unsigned long& size, unsigned char* ptData)
{
	
	// get the actual length of the compressed data
	size = atoi(getData("block_size"));
    string a = "";
    a = row_.at(8);
    
	if ( (size>0) && a.c_str())
	{
		memcpy((void*)ptData,(void*)(a.c_str()),size);
		return true;
	}
	else
	{
		ptData = 0;
		return false;
	}
}

bool TeSQLite::inClauseValues(const string& query, const string& attribute, vector<string>& inClauseVector)
{
	int i = 0, numRows = 0, count = 0, chunk = 200;
	string id, inClause = "(", plic = "'";

	TeDatabasePortal *pt = getPortal();
	if(pt->query(query) == false)
	{
		delete pt;
		return false;
	}

	while (numRows < pt->numRows())
	{
		if(count < chunk && pt->fetchRow())
		{
			id = pt->getData(attribute);
			inClause = inClause + plic + id + "',";
			++count;
			++numRows;
		}
		else
		{
			inClause = inClause.substr(0, inClause.size()-1);
			inClause += ")";
			inClauseVector.push_back(inClause);
			count = 0;
			++i;
			inClause = "(";
		}
	}
	if (numRows == 0)
	{
		delete pt;
		return true;
	}
	else
	{
		inClause = inClause.substr(0, inClause.size()-1);
		inClause += ")";
		inClauseVector.push_back(inClause);
	}

	delete pt;
	return true;
}

string TeSQLite::getSQLTime(TeTime& time)
{
	string result = "'"+ time.getDateTime("YYYYsMMsDDsHHsmmsSS", "-", ":") +"'";
	return result;
}

string TeSQLite::getSQLTemporalWhere(int time1, int time2, TeChronon chr, TeTemporalRelation rel, 
						   const string& initialTime, const string& finalTime)
{
	
	// In MySQL the first day of the week is monday with index 0
    // But in SQLite the first day in the week is sunday with index 0
    /*
	if(chr==TeDAYOFWEEK)
	{
		time1 = time1-1;
		if(time1<0)
			time1=6;
		time2 = time2-1;
		if(time2<0)
			time2=6;
	}
    */
	return(TeDatabase::getSQLTemporalWhere(time1, time2, chr, rel, initialTime, finalTime));
}
		
/**
 * Concatenate expression in SQLite
 */
string TeSQLite::concatValues(vector<string>& values, const string& unionString)
{
	string concat = " ( ";
	
	for(unsigned int i = 0; i < values.size(); ++i)
	{
		if(i != 0)
		{
			concat += " || ";

			if(!unionString.empty())
			{
				concat += "'";
				concat += unionString;
				concat += "'";
				concat += " || ";
			}
		}

		concat += values[i];
	}

	concat += " ) ";

	return concat;
}

string TeSQLite::toUpper(const string& value)
{
	string result  = "upper(";
	       result += value;
		   result += ")";

	return result;
}

/***********************/

bool TeSQLite::createDatabaseTable()
{
	string create = "CREATE TABLE te_database (";
	create += "db_version		TEXT NOT NULL PRIMARY KEY,";
	create += "db_creation		TEXT)";
	return execute (create);
}


bool TeSQLite::createProjectionTable ()
{
	string create = "CREATE TABLE te_projection (";
	create += "projection_id	INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,";
	create += "name				TEXT NOT NULL,";
	create += "long0			REAL DEFAULT '0.0',";
	create += "lat0				REAL DEFAULT '0.0',";
	create += "offx				REAL DEFAULT '0.0',";
	create += "offy				REAL DEFAULT '0.0',";
	create += "stlat1			REAL DEFAULT '0.0',";
	create += "stlat2			REAL DEFAULT '0.0',";
	create += "unit				TEXT NOT NULL,";
	create += "scale			REAL DEFAULT '0.0',";
	create += "hemis			INTEGER NOT NULL,";
	create += "datum			TEXT NOT NULL,";
	create += "radius			REAL DEFAULT '0.0',";
	create += "flattening		REAL DEFAULT '0.0',";
	create += "dx				REAL DEFAULT '0.0',";
	create += "dy				REAL DEFAULT '0.0',";
	create += "dz				REAL DEFAULT '0.0')";
	return execute (create);
}

bool TeSQLite::createLayerTable ()
{
	cout << "createlayertable" << endl;
	string create = "CREATE TABLE te_layer (";
	create += "layer_id			INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,";
	create += "projection_id	INTEGER NOT NULL,";
	create += "name				TEXT NOT NULL,";
	create += "lower_x			REAL DEFAULT '0.0' NOT NULL,";
	create += "lower_y			REAL DEFAULT '0.0' NOT NULL,";
	create += "upper_x			REAL DEFAULT '0.0' NOT NULL,";
	create += "upper_y			REAL DEFAULT '0.0' NOT NULL,";
	create += "initial_time		TEXT,";
	create += "final_time		TEXT) ";
	return execute (create);
}

bool TeSQLite::createLayerTableTable()
{
	cout << "createlayertabletable" << endl;
	string create = "CREATE TABLE te_layer_table (";
	create += "table_id				INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,";
	create += "layer_id				INTEGER,";
	create += "attr_table			TEXT NOT NULL,";
	create += "unique_id			TEXT, ";
	create += "attr_link			TEXT, ";
	create += "attr_initial_time	TEXT, ";
	create += "attr_final_time		TEXT, ";
	create += "attr_time_unit		INTEGER, ";
	create += "attr_table_type		INTEGER, ";
	create += "user_name			TEXT,";
	create += "initial_time			TEXT,";
	create += "final_time			TEXT) ";
	return execute (create);
}

//////

bool TeSQLite::createTablesRelationTable()
{
	string create = "CREATE TABLE te_tables_relation (";
	create += "relation_id			INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,";
	create += "related_table_id		INTEGER NOT NULL,";
	create += "related_attr			TEXT NOT NULL,";
	create += "external_table_name	TEXT NOT NULL,";
	create += "external_attr		TEXT NOT NULL)";
	return execute (create);
}

bool TeSQLite::createRasterMetadataTable(const string& tableName)
{
	if (tableName.empty())
		return false;

	string create = "CREATE TABLE " + tableName + " (";
	create += " geom_id				INTEGER NOT NULL, ";
	create += " band_id				INTEGER NOT NULL, ";
	create += " min_value			REAL DEFAULT '0.0', ";
	create += " max_value			REAL DEFAULT '0.0', ";
	create += " num_bits			INTEGER, ";
	create += " data_type			INTEGER, ";
	create += " photometric_type	INTEGER, ";
	create += " compression_type	INTEGER, ";
	create += " dummy				REAL, ";
     create += " PRIMARY KEY (geom_id, band_id))";
	return execute (create);
}


bool TeSQLite::createViewTable ()
{
	string create = "CREATE TABLE te_view (";
	create += "view_id			INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,";
	create += "projection_id	INTEGER NOT NULL,";
	create += "name				TEXT NOT NULL,";
	create += "user_name		TEXT,";
	create += "visibility		INTEGER)";
	return execute (create);
}

bool TeSQLite::createThemeTable ()
{
	string create = "CREATE TABLE te_theme (";
	create += " theme_id			INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,";
	create += " layer_id			INTEGER,";
	create += " view_id				INTEGER NOT NULL,";
	create += " name				TEXT NOT NULL,";
	create += " parent_id			INTEGER NOT NULL,";
	create += " priority			INTEGER NOT NULL,";
	create += " node_type			INTEGER NOT NULL,";
	create += " min_scale			REAL,";
	create += " max_scale			REAL,";
	create += " generate_attribute_where	TEXT,";
	create += " generate_spatial_where		TEXT,";
	create += " generate_temporal_where		TEXT,";
	create += " collection_table			TEXT,";
	create += " visible_rep					INTEGER,";
	create += " enable_visibility			INTEGER)";
    
    string index = "CREATE INDEX view_id_idx ON te_theme ";
    index += "(view_id)";

    if ( !execute (create) )
    {
        return false;
    }

    if ( !execute (index) )
    {
        return false;
    }

	return true;
}

bool TeSQLite::createGroupingTable()
{
	string create = "CREATE TABLE te_grouping (";
	create += " theme_id			INTEGER NOT NULL PRIMARY KEY,";
	create += " grouping_number		INTEGER,";
	create += " grouping_attr		TEXT,";
	create += " grouping_attr_type	INTEGER,";
	create += " grouping_mode		INTEGER,";
	create += " grouping_norm_attr	TEXT,";
	create += " grouping_std_dev	REAL DEFAULT '0.0',";
	create += " grouping_precision	INTEGER,";
	create += " grouping_function	TEXT,";
	create += " grouping_chronon	INTEGER)";
	return execute (create);
}

bool TeSQLite::createThemeTablesTable()
{
	string create = "CREATE TABLE te_theme_table  (";
	create += " theme_table_id		INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,";
	create += " theme_id			INTEGER NOT NULL,";
	create += " table_id			INTEGER NOT NULL,";
	create += " relation_id			INTEGER,";
	create += " table_order			INTEGER)";

    string index = "CREATE INDEX theme_table_index ON te_theme_table ";
    index += "(theme_id)";

    if ( !execute (create) )
    {
        return false;
    }

    if ( !execute (index) )
    {
        return false;
    }

	return true;
}

bool TeSQLite::createLegendTable ()
{
	string create = "CREATE TABLE te_legend (";
	create += " legend_id		INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,";
	create += " theme_id		INTEGER NOT NULL,";
	create += " group_id		INTEGER ,";
	create += " num_objs		INTEGER,";
	create += " lower_value		TEXT,";
	create += " upper_value		TEXT,";
	create += " label			TEXT) ";

    string index = "CREATE INDEX legend_index ON te_legend ";
    index += "(theme_id)";

    if ( !execute (create) )
    {
        return false;
    }

    if ( !execute (index) )
    {
        return false;
    }

	return true;
}


bool TeSQLite::createVisualTable()
{
	string create = "CREATE TABLE te_visual (";
	create += " legend_id		INTEGER NOT NULL,";
	create += " geom_type		INTEGER NOT NULL,";
	create += " lib_name		TEXT, ";
	create += " symb_id			INTEGER, ";
	create += " red				INTEGER,";
	create += " green			INTEGER,";
	create += " blue			INTEGER,";
	create += " transparency	INTEGER,";
	create += " width			INTEGER,";
	create += " contour_lib_name	TEXT, ";
	create += " contour_symb_id		INTEGER, ";
	create += " contour_red			INTEGER,";
	create += " contour_green		INTEGER,";
	create += " contour_blue		INTEGER,";
	create += " contour_transp		INTEGER,";
	create += " contour_width		INTEGER,";
	create += " size_value			INTEGER,";
	create += " pt_angle			INTEGER,";
	create += " family				TEXT,";
	create += " bold				INTEGER,";
	create += " italic				INTEGER,";
	create += " alignment_vert		REAL,";
	create += " alignment_horiz		REAL,";
	create += " tab_size			INTEGER,";
	create += " line_space			INTEGER,";
	create += " fixed_size			INTEGER,";
    create += " PRIMARY KEY (legend_id, geom_type))";
	return execute (create);
}

bool TeSQLite::createVisualRasterTable()
{
	string create = "CREATE TABLE te_visual_raster (";
	create += " theme_id		INTEGER NOT NULL,";
	create += " band_in			INTEGER NOT NULL,";
	create += " band_out		INTEGER, ";
	create += " transf_type		INTEGER, ";
	create += " param1			REAL,";
	create += " param2			REAL,";
	create += " lut_table		TEXT, ";
    create += " PRIMARY KEY (theme_id, band_in))";
	return execute (create);
}


/***********************/

bool TeSQLite::createRepresentationTable ()
{
	string create = "CREATE TABLE te_representation (";
	create += "repres_id			INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT, ";
	create += "layer_id				INTEGER NOT NULL, ";
	create += "geom_type			INTEGER NOT NULL, ";
	create += "geom_table			TEXT NOT NULL, ";
	create += "description			TEXT, ";
	create += "lower_x				REAL DEFAULT '0.0', ";
	create += "lower_y				REAL DEFAULT '0.0', ";
	create += "upper_x				REAL DEFAULT '0.0', ";
	create += "upper_y				REAL DEFAULT '0.0', ";
	create += "res_x				REAL DEFAULT '0.0', ";
	create += "res_y				REAL DEFAULT '0.0', ";
	create += "num_cols				INTEGER, ";
	create += "num_rows				INTEGER, ";
	create += "initial_time			TEXT, ";
	create += "final_time			TEXT) ";

    string index = "CREATE INDEX representation_index ON te_representation ";
    index += "(layer_id,geom_type)";

    if ( !execute (create) )
    {
        return false;
    }

    if ( !execute (index) )
    {
        return false;
    }
    
	return true;
}

bool TeSQLite::createRasterGeometry(const string& tableName)
{
	if (tableName.empty())
		return false;

	string create = "CREATE TABLE " + tableName + " (";
	create += "geom_id			INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT, ";
	create += "object_id		TEXT NOT NULL, ";
	create += "raster_table		TEXT NOT NULL, ";
	create += "lut_table		TEXT, ";
	create += "res_x			REAL DEFAULT '0.0', ";
	create += "res_y			REAL DEFAULT '0.0', ";
	create += "num_bands		INTEGER, ";
	create += "num_cols			INTEGER, ";
	create += "num_rows			INTEGER, ";
	create += "block_height		INTEGER, ";
	create += "block_width		INTEGER, ";
	create += "lower_x			REAL DEFAULT '0.0', ";
	create += "lower_y			REAL DEFAULT '0.0', ";
	create += "upper_x			REAL DEFAULT '0.0', ";
	create += "upper_y			REAL DEFAULT '0.0', ";
	create += "tiling_type      INTEGER) ";
	
    string index = "CREATE INDEX " + tableName + "_index ON " + tableName + " ";
    index += "(object_id)";

    if ( !execute (create) )
    {
        return false;
    }

    if ( !execute (index) )
    {
        return false;
    }

	return true;
}

