/************************************************************************************
TerraLib - a library for developing GIS applications.
Copyright  2001-2004 INPE and Tecgraf/PUC-Rio.

This code is part of the TerraLib library.
This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

You should have received a copy of the GNU Lesser General Public
License along with this library.

The authors reassure the license terms regarding the warranties.
They specifically disclaim any warranties, including, but not limited to,
the implied warranties of merchantability and fitness for a particular purpose.
The library provided hereunder is on an "as is" basis, and the authors have no
obligation to provide maintenance, support, updates, enhancements, or modifications.
In no event shall INPE and Tecgraf / PUC-Rio be held liable to any party for direct,
indirect, special, incidental, or consequential damages arising out of the use
of this library and its documentation.
*************************************************************************************/

/**
 * @file TePostgreSQL.h
 * @brief This file contains the especifics routines of SQLite.
 * @author Joo Borges <joaoborges@mossol.com>    
 */

/**
 * PS.: For this code works properly, you must to compile SQLite with 
 * SQLITE_ENABLE_COLUMN_METADATA preprocessor symbol defined, one way to do this is add 
 * the following line in the file sqliteInt.h, in the src directory, before compiling.
 * 
 * #define SQLITE_ENABLE_COLUMN_METADATA
 * 
 */

#ifndef  __TERRALIB_INTERNAL_SQLITE_H
#define  __TERRALIB_INTERNAL_SQLITE_H

#ifdef WIN32
#include <winsock.h>
#endif

// SQLite include files
#include <sqlite3.h>

// TerraLib include files
#include <TeTable.h>
#include <TeDatabase.h>
#include <TeDatabaseFactory.h>

#ifdef USE_STYLE_LIB
#include <TeStyle.h>
#endif


//! A concrete implementation of a driver to the SQLite SGDB
class TeSQLite : public TeDatabase
{
friend class TeSQLitePortal;

public: 

	TeSQLite();

	~TeSQLite();

	sqlite3* getSQLite () 
	{
        return sqlite3_;
    }

    //bool initDatabaseHistory();

    //bool registerDatabase(string db);

	string  escapeSequence(const string& from);

	bool newDatabase(const string& database, const string& user, const string& password, const string& host, const int &port=0, bool terralibModel=true);

	bool connect (const string& host, const string& user, const string& password, const string& database, int port=0);

	bool showDatabases (const string& host, const string& user, const string& password, vector<string>& dbNames, int port=0);

	void close();

	bool tableExist(const string& table);

	virtual bool createTable(const string& table, TeAttributeList &attr);

	bool columnExist(const string& table, const string& column, TeAttribute& attr);

	bool addColumn (const string& table,TeAttributeRep &rep);

	bool defineIntegrity(void);

	bool createRelation (const string& /* relName */, const string& /* table */, const string& /* fieldName */,
						const string& /* relatedTable */, const string& /* relatedField */, bool /* cascadeDeletion */)
	{ return true; }

	TeDBRelationType existRelation(const string& /* tableName */, const string& /* relName */)

	{	return TeNoRelation; }

	bool execute ( const string &q);

	TeDatabasePortal* getPortal ();

	string	errorMessage ();

	int	errorNum ();
	
	bool createPolygonGeometry (const string&  table);
	bool createLineGeometry (const string&  table);
	bool createRasterTable (const string& table);

// specific TerraLib database access methods

	bool insertProjection (TeProjection *proj);
	bool insertProject (TeProject *proj) { }; // TODO: implementar. funcao nova na  versao 3.2 da TerraLib
	bool insertLayer(TeLayer* layer);
	bool insertRepresentation(int layerId, TeRepresentation& rep);	
	bool insertTableInfo (int layerId, TeTable &table, const string& user="");
	bool insertRelationInfo(const int tableId, const string& tField,
						    const string& eTable, const string& eField, int& relId);
	bool insertView (TeView *view);
	bool insertViewTree (TeViewTree *tree);	
	bool insertTheme(TeTheme *theme);
	bool insertThemeGroup (TeViewTree* tree);
	bool generateLabelPositions (TeTheme *theme);
	bool insertThemeTable (int themeId, int tableId, int relationId, int tableOrder);
	bool insertLegend (TeLegendEntry *legend);
	bool insertPolygon	(const string& table, TePolygon &p);	
	bool updatePolygon	(const string& table, TePolygon &p);
	bool insertLine (const string&  table, TeLine2D &l);
	bool updateLine (const string& table, TeLine2D &l);
	bool insertPoint (const string& table, TePoint &p);
	bool insertText (const string& table, TeText &t);
	bool insertArc (const string& table,TeArc &arc);
	bool insertNode	(const string& table, TeNode &node);	
	bool insertCell	(const string& table, TeCell &c);
	int  insertId();

	bool insertRasterBlock(const string& table, const string& blockId, const TeCoord2D& ll, const TeCoord2D& ur, unsigned char *buf,unsigned long size, int band=0, unsigned int res=1, unsigned int subband=0);

	string getSQLTime(TeTime& time);
	string getSQLTemporalWhere(int time1, int time2, TeChronon chr, TeTemporalRelation rel, 
						   const string& initialTime, const string& finalTime);

	bool inClauseValues(const string& query, const string& attribute, vector<string>& inClauseVector);

	bool insertBlob (const string& tableName, const string& columnBlob, TeAttributeRep& columnId, const string& valueId, const string& fileName);
	bool insertBlob (const string& tableName, const string& columnBlob, 
		TeAttributeRep& columnId, const string& valueId, unsigned char* data, int size);

	bool insertBlob (const string& tableName, const string& columnBlob, const string& whereClause, unsigned char* data, int size);
	bool insertBlob (const string& tableName, const string& columnBlob, const string& whereClause, const string& fileName);

	//! Concat values in a vector using unionString as the join between each value
	string concatValues(vector<string>& values, const string& unionString);

	//! Returns the SQL function for upper case
	string toUpper(const string& value);

	//! Methods that create tables 
	bool createProjectTable() {} ; // TODO: implement it
	bool createRasterGeometry(const string& tableName);
	
    /////
    bool createDatabaseTable(); 
	bool createProjectionTable ();
	bool createLayerTable ();
	bool createRepresentationTable ();
	bool createViewTable ();
	bool createThemeTable ();
	bool createGroupingTable();
	bool createThemeTablesTable();
	bool createLegendTable ();
	bool createVisualTable();
	bool createVisualRasterTable();
	bool createLayerTableTable();
	bool createTablesRelationTable(); 
    bool createRasterMetadataTable(const string& tableName);

#ifdef USE_STYLE_LIB
       bool createStyleModelTables();
       bool createStyleTable(string tabNameSuffix);
       bool insertStyle(const string& table, TeStyle& style);
       bool updateStyle(const string& table, TeStyle& style);
       bool insertVisual(TeLegendEntry *legend);
       bool insertVisual(TeLegendEntry *legend, TeGeomRep rep);
       void loadStyleLibraries(TeStyleLibraryVector& libs);
       void updateStyleLibrary(TeStyleLibrary* lib);
       void deleteStyle(string tableName, int libId);
       void deleteStyleLibrary(string libName);       
#endif 

private :

	sqlite3	*sqlite3_;       // database handler
    sqlite3 *sqlite3_temp;   // temporary database handler
	char*   bufferBlob_;	 // internal buffer used to write blob data
	long	bufferBlobSize_; // size of the data already allocated
    string  sqliteDir;       // the directory default for the SQLite databases
    string  dbHistoryName;   // the name of the database and table to store the list of availables databases

};

//! A concrete implementation of a portal to a SQLite database
class TeSQLitePortal  : public TeDatabasePortal
{
private:

	sqlite3			*sqlite3_;  // database handler
    sqlite3_stmt    *result_;   // statement of the result compiled query
	//char            **row_;     // array with the row of the current result query
    vector<string>    row_;     // 

public :

// Constructor / Destructor

	TeSQLitePortal ( TeSQLite *m);

	TeSQLitePortal ( const TeSQLitePortal& p);

	~TeSQLitePortal ();

// Basic database methods

	int  insertId();

	bool query (const string &qry, TeCursorLocation l = TeSERVERSIDE, TeCursorType t = TeUNIDIRECTIONAL, TeCursorEditType e = TeREADONLY, TeCursorDataType dt = TeTEXTCURSOR );

	bool fetchRow ();

	bool fetchRow (int i);

	void freeResult ();

	string	errorMessage ();

	int	errorNum ();

// specific SQL SELECT command methods

	char*		getData (int i);
	char*		getData (const string& s);
	double		getDouble (int i);
	double		getDouble (const string& s);
	int			getInt (int i);
	int			getInt (const string& s);
	bool		getBool (const string& s);
	bool		getBool (int i);

	bool fetchGeometry(TePolygon& pol);
	bool fetchGeometry(TeLine2D& line);
	bool fetchGeometry(TeNode& n);
	bool fetchGeometry(TePoint& p);
	
	bool setData (int /* field */, string /* val */)
	{
        return true;
    }
	
    sqlite3_stmt* result()
	{
        return result_;
    }

	bool getBlob (const string& s, unsigned char* &data, long& size);
	bool getRasterBlock(unsigned long& size, unsigned char* ptData);

	TeTime getDate (int i);
	TeTime getDate (const string& s);

	string getDateAsString(int i);
	string getDateAsString(const string& s);

#ifdef USE_STYLE_LIB
   virtual void getVisual( TeVisual& vis );
#endif   

protected:
	TeLinearRing getLinearRing (int &ni);
};


/**
 * @brief This is the class for TeSQLite driver factory.
 * @author Joao Borges <joaoborges@mossol.com>
 * @ingroup DatabaseUtils
 */
class TeSQLiteFactory : public TeDatabaseFactory
{
  public :
      
    /**
     * Default constructor
     */
    TeSQLiteFactory() : TeDatabaseFactory( std::string( "SQLite" ) ) {};      
      
    /**
     * Default Destructor
     */
    ~TeSQLiteFactory() {};
      
  protected :  
  
    /**
     * Implementation for the abstract TeFactory::build.
     *
     * @param arg A const reference to the parameters used by the
     * database.
     * @return A pointer to the new generated database instance.
     */
    TeDatabase* build( const TeDatabaseFactoryParams& arg )
    {
      TeSQLite* instance_ptr = new TeSQLite();
      
      if( arg.host_ != "" ) {
        instance_ptr->connect( arg.host_, arg.user_, arg.password_,
          arg.database_, arg.port_ );
      }
  
      return (TeDatabase*)instance_ptr;
    }
};

namespace {
  static TeSQLiteFactory TeSQLiteFactory_instance;
}; 

#endif

